﻿using System;
using Template.Business.ComponentInterfaces;
using Template.Business.Components;
using Template.Business.Enumerations;
using Template.Business.ViewInterfaces;

namespace Template.Business.Presenters
{
    /// <summary>
    /// Marker interface for presenter class
    /// </summary>
    public interface IPresenter{} 

    /// <summary>
    /// The base class for all presenters
    /// </summary>
    public abstract class BasePresenter<TView>: IPresenter where TView : class, IView
    {
        private TView _view;
        protected TView View
        {
            set
            {
                if (value == null)
                    throw new ArgumentException("View cannot be set to null");
                _view = value;
            }
            get
            {
                return _view;
            }

        }

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="BasePresenter&lt;TView&gt;"/> class.
        /// </summary>
        /// <param name="view">The view.</param>
        protected BasePresenter(TView view)
        {
            View = view;
            if (CheckAuthentication(view as IPageView))
            {
                View.Load += OnViewLoad;
                SubscribeViewToEvents();
            }
        }

        #endregion

        #region Shared External Dependencies

        private IReferenceDataComponent _ReferenceDataComponent;
        protected IReferenceDataComponent ReferenceDataComponent
        {
            get
            {
                if (_ReferenceDataComponent == null)
                    _ReferenceDataComponent = ServiceLocator.Instance().Resolve<IReferenceDataComponent>();
                return _ReferenceDataComponent;
            }
        }


        private ISessionComponent _SessionComponent;
        protected ISessionComponent SessionComponent
        {
            get
            {
                if (_SessionComponent == null)
                    _SessionComponent = ServiceLocator.Instance().Resolve<ISessionComponent>();
                return _SessionComponent;
            }
        }


        private INavigationComponent _NavigationComponent;
        protected INavigationComponent NavigationComponent
        {
            get
            {
                if (_NavigationComponent == null)
                    _NavigationComponent = ServiceLocator.Instance().Resolve<INavigationComponent>();
                return _NavigationComponent;
            }
        }


        private ISecurityComponent _SecurityComponent;
        private ISecurityComponent SecurityComponent
        {
            get
            {
                if (_SecurityComponent == null)
                    _SecurityComponent = ServiceLocator.Instance().Resolve<ISecurityComponent>();
                return _SecurityComponent;
            }
        }
	
	
	

        #endregion

        #region Abstract Methods

        /// <summary>
        /// Subscribes the view to events.
        /// This method has to be implemented in every presenter class
        /// </summary>
        protected abstract void SubscribeViewToEvents();

        /// <summary>
        /// Called when the view is loaded.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected abstract void OnViewLoad(object sender, EventArgs e);

        /// <summary>
        /// Method that should be called to bind the model to the view
        /// </summary>
        protected abstract void LoadViewFromModel();

        #endregion

        private bool CheckAuthentication(IPageView page)
        {
            if (page==null)//we have a control instead of page
                return true;
            if (SecurityComponent.IsPublic(page.PageIdentifier) || SessionComponent.IsLoggedIn)
                return true;
            //not logged in ! go to logonpage
            NavigationComponent.NavigateAndReturnTo(PageIdentifier.LogonPage);
            return false;
        }


        /// <summary>
        /// Indicating whether the view was originated due to post back.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is postback; otherwise, <c>false</c>.
        /// </value>
        protected bool IsPostBack
        {
            get { return View.IsPostBack; }
        }

        /// <summary>
        /// Gets a parameter from the querystring.
        /// </summary>
        /// <param name="parameter">The named querystring parameter.</param>
        /// <returns></returns>
        protected string GetQueryParameter(QueryStringParameter parameter)
        {
            return View.QueryString[parameter.ToString()];    
        }

        /// <summary>
        /// Gets a parameter from the querystring.
        /// </summary>
        /// <param name="parameter">The named querystring parameter as an integer. 0 if no value is found</param>
        /// <returns></returns>
        protected int GetQueryParameterAsInt(QueryStringParameter parameter)
        {
            int result;
            int.TryParse(View.QueryString[parameter.ToString()], out result);
            return result;
        }



    }

}