﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Practices.Unity;
using Template.Business.ComponentInterfaces;
using Template.Business.Components;
using Template.Data.DataAccess;
using Template.Data.Interfaces;

namespace Template.Business
{
    /// <summary>
    /// The service locator will use an IoC to resolve the dependencies on the external components 
    /// </summary>
    public class ServiceLocator
    {
        private static ServiceLocator _instance = null;
        private UnityContainer _container = null;

        /// <summary>
        /// Non public constructor.  Use Instance() instead.
        /// </summary>
        private ServiceLocator()
        {
        }

        /// <summary>
        /// Returns the one and only instance of the Service Locator
        /// </summary>
        /// <returns>The one and only instance</returns>
        public static ServiceLocator Instance()
        {
            if (null == _instance)
            {
                _instance = new ServiceLocator();
            }

            return _instance;
        }

        /// <summary>
        /// Initializes the container for Application usage 
        /// </summary>
        public void InitializeForApplication()
        {
            _container = new UnityContainer();
            // Register the concrete classes for our business components
            _container.RegisterType<IReferenceDataComponent, ReferenceDataComponent>();
            _container.RegisterType<ISecurityComponent, SecurityComponent>();

            // Register the concrete classes for our data access classes
            _container.RegisterType<IReferenceDataAccess, ReferenceDataAccess>();

        }


        /// <summary>
        /// Initialize the container for mocking & unittests
        /// </summary>
        internal void InitializeForUnitTesting()
        {
            if (_container != null)
                _container.Dispose();
            _container = new UnityContainer();
        }

        /// <summary>
        /// internal registertype method for Mocking & unit testing
        /// use this to populate the container with mocked objects
        /// </summary>
        /// <typeparam name="TInterface"></typeparam>
        internal void RegisterInstance<TInterface>(TInterface instance)
        {
            if (_container == null)
                throw new InvalidOperationException("The container has not been initialized yet. Call .Initialize() first");
            _container.RegisterInstance<TInterface>(instance);
        }

        /// <summary>
        /// registers a new type with the container.
        /// </summary>
        /// <typeparam name="TInterface">The type of the interface.</typeparam>
        /// <typeparam name="TClass">The type of the class.</typeparam>
        public void RegisterType<TInterface, TClass>() where TClass : TInterface
        {
            if (_container == null)
                throw new InvalidOperationException("The container has not been initialized yet. Call .Initialize() first");
            _container.RegisterType<TInterface, TClass>();
        }

        /// <summary>
        /// Get an instance of the default requested type from the container.
        /// </summary>
        /// <typeparam name="T">System.Type of object to get from the container.</typeparam>
        /// <returns>The retrieved object.</returns>
        public T Resolve<T>()
        {
            if (_container == null)
                throw new InvalidOperationException("The container has not been initialized yet. Call .Initialize() first");
            T result = _container.Resolve<T>();
            return result;
        }

    }
}
