﻿
using System;
using System.Collections.Specialized;
using System.Web.UI;
using Template.Business.ComponentInterfaces;
using Template.Business.Presenters;
using Template.Business.ViewInterfaces;
using Template.Business.Enumerations;

namespace Template.Web.Code
{
    public abstract class BasePage : Page, IPageView
    {
        /// <summary>
        /// Raises the <see cref="E:System.Web.UI.Control.Init"/> event to initialize the page.
        /// </summary>
        /// <param name="e">An <see cref="T:System.EventArgs"/> that contains the event data.</param>
        protected override void OnInit(EventArgs e)
        {
            CreatePresenter();
            CreateEventhandlers();
            base.OnInit(e);
        }

        /// <summary>
        /// Inits the presenter. This method should just instanciate and return the page's presenter. 
        /// The presenter will automatically be kept alive due to subscribing to the view's Load-event
        /// </summary>
        protected abstract IPresenter CreatePresenter();


        /// <summary>
        /// Creates the event handlers for the page generated events
        /// Handlers should be of type : ButtonTest.Click += (s, e) => TestButtonClick(null,null);
        /// where TestButtonClick is an event defined on the view,  and ButtonTest is a Button control on the page
        /// </summary>
        protected abstract void CreateEventhandlers();


        #region IPageView Members

        /// <summary>
        /// Gets the querystring parameters.
        /// </summary>
        /// <value>The querystring parameters.</value>
        public NameValueCollection QueryString
        {
            get { return Request.QueryString; }
        }

        /// <summary>
        /// The navigationId of the View
        /// </summary>
        /// <value></value>
        public abstract PageIdentifier PageIdentifier { get; }

        #endregion

        /// <summary>
        /// Loads the usercontrol specified by the controlIdentifier.
        /// The control will have a server generated Id
        /// </summary>
        /// <param name="controlIdentifier">The control identifier.</param>
        /// <returns></returns>
        internal BaseUserControl LoadUserControl(ControlIdentifier controlIdentifier)
        {
            return LoadUserControl(controlIdentifier, null);
        }

        /// <summary>
        /// Loads the usercontrol specified by the controlIdentifier.
        /// </summary>
        /// <param name="controlIdentifier">The control identifier.</param>
        /// <param name="Id">The id of the control. Leave null or empty for servergenerated Id</param>
        /// <returns></returns>
        internal BaseUserControl LoadUserControl(ControlIdentifier controlIdentifier,string Id)
        {
            BaseUserControl result;
            result = this.LoadControl(new NavigationComponent().GetUrl(controlIdentifier)) as BaseUserControl;
            if (result == null)
                throw new InvalidOperationException("Error load usercontrol '" + controlIdentifier +
                                                "' - pleasy verify ControlIdentifier mapping and that BaseUserControl is subclassed");
            if (!string.IsNullOrEmpty(Id))
                result.ID = Id;
            return result;
        }

        #region Page parameters

        /// <summary>
        /// Sets a page parameter that will be persisted across postback.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <param name="value">The value.</param>
        internal void SetPageParam(string key, string value)
        {
            base.ViewState[key] = value;
        }

        /// <summary>
        /// Sets a page parameter that will be persisted across postback.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <param name="value">The value.</param>
        internal void SetPageParam(string key, bool value)
        {
            SetPageParam(key, value.ToString());
        }

        /// <summary>
        /// Sets a page parameter that will be persisted across postback.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <param name="value">The value.</param>
        internal void SetPageParam(string key, int value)
        {
            SetPageParam(key, value.ToString());
        }

        /// <summary>
        /// Gets a page parameter
        /// </summary>
        /// <param name="key">The key.</param>
        /// <returns></returns>
        internal string GetPageParam(string key)
        {
            string result = default(string);
            foreach (string k in base.ViewState.Keys)
                if (k == key)
                    result = base.ViewState[k] as string;
            return result;
        }

        /// <summary>
        /// Gets a boolean page parameter
        /// </summary>
        /// <param name="key">The key.</param>
        /// <returns></returns>
        internal bool GetPageParamAsBool(string key)
        {
            bool result;
            bool.TryParse(GetPageParam(key), out result);
            return result;
        }

        /// <summary>
        /// Gets an integer page parameter
        /// </summary>
        /// <param name="key">The key.</param>
        /// <returns></returns>
        internal int GetPageParamAsInt(string key)
        {
            int result;
            int.TryParse(GetPageParam(key), out result);
            return result;
        }


        #endregion
    }
}