﻿
using System.Collections.Specialized;
using System.Web.UI;
using System.Web.UI.WebControls;
using Template.Business.ComponentInterfaces;
using Template.Business.Enumerations;
using Template.Business.Presenters;
using Template.Business.ViewInterfaces;
using System;

namespace Template.Web.Code
{
    public abstract class BaseUserControl : UserControl, IControlView
    {
        private bool _wasPreviouslyLoaded = false;
        
        /// <summary>
        /// Gets a reference to the <see cref="T:Template.Web.Code.BasePage"/> instance that contains the server control.
        /// </summary>
        /// <value></value>
        /// <returns>
        /// The <see cref="T:Template.Web.Code.BasePage"/> instance that contains the server control.
        /// </returns>
        /// <exception cref="T:System.InvalidOperationException">
        /// The control is a <see cref="T:System.Web.UI.WebControls.Substitution"/> control.
        /// </exception>
        protected new BasePage Page
        {
            get { return base.Page as BasePage; }
        }
        
        /// <summary>
        /// Raises the <see cref="E:System.Web.UI.Control.Init"/> event to initialize the page.
        /// </summary>
        /// <param name="e">An <see cref="T:System.EventArgs"/> that contains the event data.</param>
        protected override void OnInit(EventArgs e)
        {
            CreatePresenter();
            CreateEventhandlers();
            base.OnInit(e);
            //trick to see if the control was previously loaded
            _wasPreviouslyLoaded = Page.GetPageParamAsBool(ControlIdentifier + "_Initialized");
            //set this to false for the next postback - when rendered this will be undone
            Page.SetPageParam(ControlIdentifier + "_Initialized", false);
        }

        /// <summary>
        /// Inits the presenter. This method should just instanciate and return the page's presenter. 
        /// The presenter will automatically be kept alive due to subscribing to the view's Load-event
        /// </summary>
        protected abstract IPresenter CreatePresenter();


        /// <summary>
        /// Creates the event handlers for the page generated events
        /// Handlers should be of type : ButtonTest.Click += (s, e) => TestButtonClick(null,null);
        /// where TestButtonClick is an event defined on the view,  and ButtonTest is a Button control on the page
        /// </summary>
        protected abstract void CreateEventhandlers();


        #region IControlView members

        /// <summary>
        /// Gets the querystring parameters.
        /// </summary>
        /// <value>The querystring parameters.</value>
        public NameValueCollection QueryString
        {
            get { return Request.QueryString; }
        }

        /// <summary>
        /// The Identifier of the Control
        /// </summary>
        /// <value></value>
        public abstract ControlIdentifier ControlIdentifier { get; }

        /// <summary>
        /// Gets a value indicating whether the user control is being loaded in response to a client postback, or if it is being loaded and accessed for the first time.
        /// </summary>
        /// <value></value>
        /// <returns>true if the user control is being re-loaded in response to a client postback; otherwise, false.
        /// </returns>
        public new bool IsPostBack
        {
              get
                {
                    return base.IsPostBack && _wasPreviouslyLoaded;
                }
        }

        #endregion

        /// <summary>
        /// Raises the <see cref="E:System.Web.UI.Control.PreRender"/> event.
        /// This will set a hidden field in the viewstate to indicate the control was loaded before
        /// This mechanism is needed for correct behavior of the IsPostBack property
        /// </summary>
        /// <param name="e">An <see cref="T:System.EventArgs"/> object that contains the event data.</param>
        protected override void OnPreRender(System.EventArgs e)
        {
            Page.SetPageParam(ControlIdentifier + "_Initialized", true);
            base.OnPreRender(e);
        }

    }
}